@@ -16,7 +16,7 @@
  *
  *	Copyright (c) 2001, PostgreSQL Global Development Group
  *
- *	$Header: /cvsroot/pgsql/src/backend/postmaster/pgstat.c,v 1.17 2002/02/07 22:20:26 tgl Exp $
+ *	$Header: /cvsroot/pgsql-server/src/backend/postmaster/pgstat.c,v 1.31.2.2 2003/07/22 19:13:25 tgl Exp $
  * ----------
  */
 #include "postgres.h"
@@ -25,7 +25,6 @@
 #include <fcntl.h>
 #include <sys/param.h>
 #include <sys/time.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -38,12 +37,11 @@
 #include "catalog/pg_shadow.h"
 #include "catalog/pg_database.h"
 #include "libpq/pqsignal.h"
-#ifdef MULTIBYTE
 #include "mb/pg_wchar.h"
-#endif
 #include "miscadmin.h"
 #include "utils/memutils.h"
 #include "storage/backendid.h"
+#include "storage/ipc.h"
 #include "utils/rel.h"
 #include "utils/hsearch.h"
 #include "utils/ps_status.h"
@@ -77,13 +75,15 @@ static int	pgStatPid;
 static long pgStatNumMessages = 0;
 
 static bool pgStatRunningInCollector = FALSE;
+
 static int	pgStatTabstatAlloc = 0;
 static int	pgStatTabstatUsed = 0;
 static PgStat_MsgTabstat **pgStatTabstatMessages = NULL;
+#define TABSTAT_QUANTUM		4	/* we alloc this many at a time */
+
 static int	pgStatXactCommit = 0;
 static int	pgStatXactRollback = 0;
 
-
 static TransactionId pgStatDBHashXact = InvalidTransactionId;
 static HTAB *pgStatDBHash = NULL;
 static HTAB *pgStatBeDead = NULL;
@@ -133,6 +133,10 @@ static void pgstat_recv_resetcounter(PgStat_MsgResetcounter *msg, int len);
  *
  *	Called from postmaster at startup. Create the resources required
  *	by the statistics collector process.
+ *
+ *	NOTE: failure exit from this routine causes the postmaster to abort.
+ *	This is unfriendly and should not be done except in dire straits.
+ *	Better to let the postmaster start with stats collection disabled.
  * ----------
  */
 int
@@ -173,8 +177,8 @@ pgstat_init(void)
 	 */
 	if ((pgStatSock = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
 	{
-		perror("PGSTAT: socket(2)");
-		return -1;
+		elog(LOG, "PGSTAT: socket() failed: %m");
+		goto startup_failed;
 	}
 
 	/*
@@ -187,17 +191,13 @@ pgstat_init(void)
 	alen = sizeof(pgStatAddr);
 	if (bind(pgStatSock, (struct sockaddr *) & pgStatAddr, alen) < 0)
 	{
-		perror("PGSTAT: bind(2)");
-		close(pgStatSock);
-		pgStatSock = -1;
-		return -1;
+		elog(LOG, "PGSTAT: bind(127.0.0.1) failed: %m");
+		goto startup_failed;
 	}
 	if (getsockname(pgStatSock, (struct sockaddr *) & pgStatAddr, &alen) < 0)
 	{
-		perror("PGSTAT: getsockname(2)");
-		close(pgStatSock);
-		pgStatSock = -1;
-		return -1;
+		elog(LOG, "PGSTAT: getsockname() failed: %m");
+		goto startup_failed;
 	}
 
 	/*
@@ -208,10 +208,8 @@ pgstat_init(void)
 	 */
 	if (connect(pgStatSock, (struct sockaddr *) & pgStatAddr, alen) < 0)
 	{
-		perror("PGSTAT: connect(2)");
-		close(pgStatSock);
-		pgStatSock = -1;
-		return -1;
+		elog(LOG, "PGSTAT: connect() failed: %m");
+		goto startup_failed;
 	}
 
 	/*
@@ -222,10 +220,8 @@ pgstat_init(void)
 	 */
 	if (fcntl(pgStatSock, F_SETFL, O_NONBLOCK) < 0)
 	{
-		perror("PGSTAT: fcntl(2)");
-		close(pgStatSock);
-		pgStatSock = -1;
-		return -1;
+		elog(LOG, "PGSTAT: fcntl() failed: %m");
+		goto startup_failed;
 	}
 
 	/*
@@ -233,12 +229,22 @@ pgstat_init(void)
 	 */
 	if (pipe(pgStatPmPipe) < 0)
 	{
-		perror("PGSTAT: pipe(2)");
-		close(pgStatSock);
-		pgStatSock = -1;
-		return -1;
+		elog(LOG, "PGSTAT: pipe() failed: %m");
+		goto startup_failed;
 	}
 
+	return 0;
+
+startup_failed:
+	if (pgStatSock >= 0)
+		close(pgStatSock);
+	pgStatSock = -1;
+
+	/* Adjust GUC variables to suppress useless activity */
+	pgstat_collect_querystring = false;
+	pgstat_collect_tuplelevel = false;
+	pgstat_collect_blocklevel = false;
+
 	return 0;
 }
 
@@ -248,6 +254,8 @@ pgstat_init(void)
  *
  *	Called from postmaster at startup or after an existing collector
  *	died.  Fire up a fresh statistics collector.
+ *
+ *	NOTE: failure exit from this routine causes the postmaster to abort.
  * ----------
  */
 int
@@ -260,12 +268,11 @@ pgstat_start(void)
 		return 0;
 
 	/*
-	 * Check that the socket at least is there
+	 * Check that the socket is there, else pgstat_init failed
 	 */
 	if (pgStatSock < 0)
 	{
-		fprintf(stderr,
-		   "PGSTAT: suppress collector startup due to missing socket\n");
+		elog(LOG, "PGSTAT: statistics collector startup skipped");
 		return 0;
 	}
 
@@ -288,9 +295,9 @@ pgstat_start(void)
 			/* Specific beos actions */
 			beos_backend_startup_failed();
 #endif
-			perror("PGSTAT: fork(2)");
+			elog(LOG, "PGSTAT: fork() failed: %m");
 			pgStatRunning = 0;
-			return -1;
+			return 0;
 
 		case 0:
 			break;
@@ -309,6 +316,8 @@ pgstat_start(void)
 
 	IsUnderPostmaster = true;	/* we are a postmaster subprocess now */
 
+	MyProcPid = getpid();		/* reset MyProcPid */
+
 	/* Lose the postmaster's on-exit routines */
 	on_exit_reset();
 
@@ -371,13 +380,12 @@ pgstat_beterm(int pid)
 {
 	PgStat_MsgBeterm msg;
 
-	if (!pgstat_collect_startcollector || pgStatSock < 0)
+	if (pgStatSock < 0)
 		return;
 
+	MemSet(&(msg.m_hdr), 0, sizeof(msg.m_hdr));
 	msg.m_hdr.m_type = PGSTAT_MTYPE_BETERM;
-	msg.m_hdr.m_backendid = 0;
 	msg.m_hdr.m_procpid = pid;
-	msg.m_hdr.m_databaseid = 0;
 
 	pgstat_send(&msg, sizeof(msg));
 }
@@ -401,7 +409,7 @@ pgstat_bestart(void)
 {
 	PgStat_MsgBestart msg;
 
-	if (!pgstat_collect_startcollector || pgStatSock < 0)
+	if (pgStatSock < 0)
 		return;
 
 	pgstat_setheader(&msg.m_hdr, PGSTAT_MTYPE_BESTART);
@@ -427,12 +435,7 @@ pgstat_report_activity(char *what)
 		return;
 
 	len = strlen(what);
-#ifdef MULTIBYTE
-	len = pg_mbcliplen((const unsigned char *)what, len, PGSTAT_ACTIVITY_SIZE - 1);
-#else
-	if (len >= PGSTAT_ACTIVITY_SIZE)
-		len = PGSTAT_ACTIVITY_SIZE - 1;
-#endif
+	len = pg_mbcliplen((const unsigned char *) what, len, PGSTAT_ACTIVITY_SIZE - 1);
 
 	memcpy(msg.m_what, what, len);
 	msg.m_what[len] = '\0';
@@ -515,14 +518,6 @@ pgstat_vacuum_tabstat(void)
 	if (pgStatSock < 0)
 		return 0;
 
-	/*
-	 * We don't vacuum inside of transaction blocks, because a possible
-	 * later rollback might reactivate objects we didn't find because of
-	 * earlier destruction in the xact.
-	 */
-	if (IsTransactionBlock())
-		return 0;
-
 	/*
 	 * If not done for this transaction, read the statistics collector
 	 * stats file into some hash tables.
@@ -582,7 +577,7 @@ pgstat_vacuum_tabstat(void)
 		if (msg.m_nentries >= PGSTAT_NUM_TABPURGE)
 		{
 			len = offsetof(PgStat_MsgTabpurge, m_tableid[0])
-				+ msg.m_nentries * sizeof(Oid);
+				+msg.m_nentries * sizeof(Oid);
 
 			pgstat_setheader(&msg.m_hdr, PGSTAT_MTYPE_TABPURGE);
 			pgstat_send(&msg, len);
@@ -597,7 +592,7 @@ pgstat_vacuum_tabstat(void)
 	if (msg.m_nentries > 0)
 	{
 		len = offsetof(PgStat_MsgTabpurge, m_tableid[0])
-			+ msg.m_nentries * sizeof(Oid);
+			+msg.m_nentries * sizeof(Oid);
 
 		pgstat_setheader(&msg.m_hdr, PGSTAT_MTYPE_TABPURGE);
 		pgstat_send(&msg, len);
@@ -611,8 +606,8 @@ pgstat_vacuum_tabstat(void)
 	dbidlist = (Oid *) palloc(sizeof(Oid) * dbidalloc);
 
 	dbrel = heap_openr(DatabaseRelationName, AccessShareLock);
-	dbscan = heap_beginscan(dbrel, 0, SnapshotNow, 0, NULL);
-	while (HeapTupleIsValid(dbtup = heap_getnext(dbscan, FALSE)))
+	dbscan = heap_beginscan(dbrel, SnapshotNow, 0, NULL);
+	while ((dbtup = heap_getnext(dbscan, ForwardScanDirection)) != NULL)
 	{
 		if (dbidused >= dbidalloc)
 		{
@@ -620,7 +615,7 @@ pgstat_vacuum_tabstat(void)
 			dbidlist = (Oid *) repalloc((char *) dbidlist,
 										sizeof(Oid) * dbidalloc);
 		}
-		dbidlist[dbidused++] = dbtup->t_data->t_oid;
+		dbidlist[dbidused++] = HeapTupleGetOid(dbtup);
 	}
 	heap_endscan(dbscan);
 	heap_close(dbrel, AccessShareLock);
@@ -739,7 +734,7 @@ pgstat_ping(void)
 void
 pgstat_initstats(PgStat_Info *stats, Relation rel)
 {
-	PgStat_TableEntry *useent = NULL;
+	PgStat_TableEntry *useent;
 	Oid			rel_id = rel->rd_id;
 	int			mb;
 	int			i;
@@ -752,39 +747,44 @@ pgstat_initstats(PgStat_Info *stats, Relation rel)
 	stats->heap_scan_counted = FALSE;
 	stats->index_scan_counted = FALSE;
 
-	if (!pgstat_collect_startcollector || pgStatSock < 0)
+	if (pgStatSock < 0)
 	{
 		stats->no_stats = TRUE;
 		return;
 	}
 
 	/*
-	 * On the first of all calls initialize the message buffers.
+	 * On the first of all calls create some message buffers.
 	 */
-	if (pgStatTabstatAlloc == 0)
+	if (pgStatTabstatMessages == NULL)
 	{
-		pgStatTabstatAlloc = 4;
-		pgStatTabstatMessages = (PgStat_MsgTabstat **)
-			malloc(sizeof(PgStat_MsgTabstat *) * pgStatTabstatAlloc);
-		if (pgStatTabstatMessages == NULL)
+		PgStat_MsgTabstat *newMessages;
+		PgStat_MsgTabstat **msgArray;
+
+		newMessages = (PgStat_MsgTabstat *)
+			malloc(sizeof(PgStat_MsgTabstat) * TABSTAT_QUANTUM);
+		if (newMessages == NULL)
 		{
-			perror("PGSTATBE: malloc(2)");
+			elog(LOG, "out of memory");
 			return;
 		}
-		for (i = 0; i < pgStatTabstatAlloc; i++)
+		msgArray = (PgStat_MsgTabstat **)
+			malloc(sizeof(PgStat_MsgTabstat *) * TABSTAT_QUANTUM);
+		if (msgArray == NULL)
 		{
-			pgStatTabstatMessages[i] = (PgStat_MsgTabstat *)
-				malloc(sizeof(PgStat_MsgTabstat));
-			if (pgStatTabstatMessages[i] == NULL)
-			{
-				perror("PGSTATBE: malloc(2)");
-				return;
-			}
+			free(newMessages);
+			elog(LOG, "out of memory");
+			return;
 		}
+		MemSet(newMessages, 0, sizeof(PgStat_MsgTabstat) * TABSTAT_QUANTUM);
+		for (i = 0; i < TABSTAT_QUANTUM; i++)
+			msgArray[i] = newMessages++;
+		pgStatTabstatMessages = msgArray;
+		pgStatTabstatAlloc = TABSTAT_QUANTUM;
 	}
 
 	/*
-	 * Lookup the so far used table slots for this relation.
+	 * Search the already-used message slots for this relation.
 	 */
 	for (mb = 0; mb < pgStatTabstatUsed; mb++)
 	{
@@ -806,7 +806,7 @@ pgstat_initstats(PgStat_Info *stats, Relation rel)
 		 */
 		i = pgStatTabstatMessages[mb]->m_nentries++;
 		useent = &pgStatTabstatMessages[mb]->m_entry[i];
-		memset(useent, 0, sizeof(PgStat_TableEntry));
+		MemSet(useent, 0, sizeof(PgStat_TableEntry));
 		useent->t_id = rel_id;
 		stats->tabentry = (void *) useent;
 		return;
@@ -817,27 +817,31 @@ pgstat_initstats(PgStat_Info *stats, Relation rel)
 	 */
 	if (pgStatTabstatUsed >= pgStatTabstatAlloc)
 	{
-		pgStatTabstatAlloc += 4;
-		pgStatTabstatMessages = (PgStat_MsgTabstat **)
-			realloc(pgStatTabstatMessages,
-					sizeof(PgStat_MsgTabstat *) * pgStatTabstatAlloc);
-		if (pgStatTabstatMessages == NULL)
+		int		newAlloc = pgStatTabstatAlloc + TABSTAT_QUANTUM;
+		PgStat_MsgTabstat *newMessages;
+		PgStat_MsgTabstat **msgArray;
+
+		newMessages = (PgStat_MsgTabstat *)
+			malloc(sizeof(PgStat_MsgTabstat) * TABSTAT_QUANTUM);
+		if (newMessages == NULL)
 		{
-			pgStatTabstatAlloc -= 4;
-			perror("PGSTATBE: malloc(2)");
+			elog(LOG, "out of memory");
 			return;
 		}
-		for (i = pgStatTabstatUsed; i < pgStatTabstatAlloc; i++)
+		msgArray = (PgStat_MsgTabstat **)
+			realloc(pgStatTabstatMessages,
+					sizeof(PgStat_MsgTabstat *) * newAlloc);
+		if (msgArray == NULL)
 		{
-			pgStatTabstatMessages[i] = (PgStat_MsgTabstat *)
-				malloc(sizeof(PgStat_MsgTabstat));
-			if (pgStatTabstatMessages[i] == NULL)
-			{
-				pgStatTabstatAlloc -= 4;
-				perror("PGSTATBE: malloc(2)");
-				return;
-			}
+			free(newMessages);
+			elog(LOG, "out of memory");
+			return;
 		}
+		MemSet(newMessages, 0, sizeof(PgStat_MsgTabstat) * TABSTAT_QUANTUM);
+		for (i = 0; i < TABSTAT_QUANTUM; i++)
+			msgArray[pgStatTabstatAlloc + i] = newMessages++;
+		pgStatTabstatMessages = msgArray;
+		pgStatTabstatAlloc = newAlloc;
 	}
 
 	/*
@@ -846,10 +850,9 @@ pgstat_initstats(PgStat_Info *stats, Relation rel)
 	mb = pgStatTabstatUsed++;
 	pgStatTabstatMessages[mb]->m_nentries = 1;
 	useent = &pgStatTabstatMessages[mb]->m_entry[0];
-	memset(useent, 0, sizeof(PgStat_TableEntry));
+	MemSet(useent, 0, sizeof(PgStat_TableEntry));
 	useent->t_id = rel_id;
 	stats->tabentry = (void *) useent;
-	return;
 }
 
 
@@ -1162,14 +1165,14 @@ pgstat_main(void)
 	 */
 	if (pipe(pgStatPipe) < 0)
 	{
-		perror("PGSTAT: pipe(2)");
+		elog(LOG, "PGSTAT: pipe() failed: %m");
 		exit(1);
 	}
 
 	switch (fork())
 	{
 		case -1:
-			perror("PGSTAT: fork(2)");
+			elog(LOG, "PGSTAT: fork() failed: %m");
 			exit(1);
 
 		case 0:
@@ -1191,6 +1194,8 @@ pgstat_main(void)
 	 */
 	pqsignal(SIGCHLD, SIG_DFL);
 
+	MyProcPid = getpid();		/* reset MyProcPid */
+
 	/*
 	 * Identify myself via ps
 	 */
@@ -1221,8 +1226,7 @@ pgstat_main(void)
 							   &hash_ctl, HASH_ELEM | HASH_FUNCTION);
 	if (pgStatBeDead == NULL)
 	{
-		fprintf(stderr,
-				"PGSTAT: Creation of dead backend hash table failed\n");
+		elog(LOG, "PGSTAT: Creation of dead backend hash table failed");
 		exit(1);
 	}
 
@@ -1233,7 +1237,7 @@ pgstat_main(void)
 							   sizeof(PgStat_StatBeEntry) * MaxBackends);
 	if (pgStatBeTable == NULL)
 	{
-		perror("PGSTAT: Allocation of backend table failed");
+		elog(LOG, "PGSTAT: Allocation of backend table failed");
 		exit(1);
 	}
 	memset(pgStatBeTable, 0, sizeof(PgStat_StatBeEntry) * MaxBackends);
@@ -1253,19 +1257,27 @@ pgstat_main(void)
 		 */
 		if (need_statwrite)
 		{
-			gettimeofday(&timeout, NULL);
-			timeout.tv_usec = next_statwrite.tv_usec - timeout.tv_usec;
-			timeout.tv_sec = next_statwrite.tv_sec - timeout.tv_sec;
-			if (timeout.tv_usec < 0)
-			{
-				timeout.tv_sec -= 1;
-				timeout.tv_usec += 1000000;
-			}
-			if (timeout.tv_sec < 0)
+			struct timeval now;
+
+			gettimeofday(&now, NULL);
+			/* avoid assuming that tv_sec is signed */
+			if (now.tv_sec > next_statwrite.tv_sec ||
+				(now.tv_sec == next_statwrite.tv_sec &&
+				 now.tv_usec >= next_statwrite.tv_usec))
 			{
 				timeout.tv_sec = 0;
 				timeout.tv_usec = 0;
 			}
+			else
+			{
+				timeout.tv_sec = next_statwrite.tv_sec - now.tv_sec;
+				timeout.tv_usec = next_statwrite.tv_usec - now.tv_usec;
+				if (timeout.tv_usec < 0)
+				{
+					timeout.tv_sec--;
+					timeout.tv_usec += 1000000;
+				}
+			}
 		}
 
 		/*
@@ -1289,7 +1301,7 @@ pgstat_main(void)
 		{
 			if (errno == EINTR)
 				continue;
-			perror("PGSTAT: select(2)");
+			elog(LOG, "PGSTAT: select() failed: %m");
 			exit(1);
 		}
 
@@ -1329,7 +1341,7 @@ pgstat_main(void)
 				{
 					if (errno == EINTR)
 						continue;
-					perror("PGSTAT: read(2)");
+					elog(LOG, "PGSTAT: read() failed: %m");
 					exit(1);
 				}
 				if (len == 0)	/* EOF on the pipe! */
@@ -1347,7 +1359,7 @@ pgstat_main(void)
 						 * sync with the buffer process somehow. Abort so
 						 * that we can restart both processes.
 						 */
-						fprintf(stderr, "PGSTAT: bogus message length\n");
+						elog(LOG, "PGSTAT: bogus message length");
 						exit(1);
 					}
 				}
@@ -1497,7 +1509,7 @@ pgstat_recvbuffer(void)
 	 */
 	if (fcntl(writePipe, F_SETFL, O_NONBLOCK) < 0)
 	{
-		perror("PGSTATBUFF: fcntl(2)");
+		elog(LOG, "PGSTATBUFF: fcntl() failed: %m");
 		exit(1);
 	}
 
@@ -1507,7 +1519,7 @@ pgstat_recvbuffer(void)
 	msgbuffer = (char *) malloc(PGSTAT_RECVBUFFERSZ);
 	if (msgbuffer == NULL)
 	{
-		perror("PGSTATBUFF: malloc()");
+		elog(LOG, "PGSTATBUFF: malloc() failed");
 		exit(1);
 	}
 
@@ -1534,7 +1546,7 @@ pgstat_recvbuffer(void)
 		{
 			if (!overflow)
 			{
-				fprintf(stderr, "PGSTATBUFF: Warning - receive buffer full\n");
+				elog(LOG, "PGSTATBUFF: Warning - receive buffer full");
 				overflow = true;
 			}
 		}
@@ -1565,7 +1577,7 @@ pgstat_recvbuffer(void)
 		{
 			if (errno == EINTR)
 				continue;
-			perror("PGSTATBUFF: select(2)");
+			elog(LOG, "PGSTATBUFF: select() failed: %m");
 			exit(1);
 		}
 
@@ -1577,11 +1589,11 @@ pgstat_recvbuffer(void)
 		{
 			fromlen = sizeof(fromaddr);
 			len = recvfrom(pgStatSock,
-						   &input_buffer, sizeof(PgStat_Msg), 0,
+						   (char *) &input_buffer, sizeof(PgStat_Msg), 0,
 						   (struct sockaddr *) &fromaddr, &fromlen);
 			if (len < 0)
 			{
-				perror("PGSTATBUFF: recvfrom(2)");
+				elog(LOG, "PGSTATBUFF: recvfrom() failed: %m");
 				exit(1);
 			}
 
@@ -1656,7 +1668,7 @@ pgstat_recvbuffer(void)
 			{
 				if (errno == EINTR || errno == EAGAIN)
 					continue;	/* not enough space in pipe */
-				perror("PGSTATBUFF: write(2)");
+				elog(LOG, "PGSTATBUFF: write() failed: %m");
 				exit(1);
 			}
 			/* NB: len < xfr is okay */
@@ -1709,7 +1721,7 @@ pgstat_add_backend(PgStat_MsgHdr *msg)
 	 */
 	if (msg->m_backendid < 1 || msg->m_backendid > MaxBackends)
 	{
-		fprintf(stderr, "PGSTAT: Invalid backend ID %d\n", msg->m_backendid);
+		elog(LOG, "PGSTAT: Invalid backend ID %d", msg->m_backendid);
 		return -1;
 	}
 
@@ -1765,7 +1777,7 @@ pgstat_add_backend(PgStat_MsgHdr *msg)
 												 HASH_ENTER, &found);
 	if (dbentry == NULL)
 	{
-		fprintf(stderr, "PGSTAT: DB hash table out of memory - abort\n");
+		elog(LOG, "PGSTAT: DB hash table out of memory - abort");
 		exit(1);
 	}
 
@@ -1794,8 +1806,8 @@ pgstat_add_backend(PgStat_MsgHdr *msg)
 									  HASH_ELEM | HASH_FUNCTION);
 		if (dbentry->tables == NULL)
 		{
-			fprintf(stderr, "PGSTAT: failed to initialize hash table for "
-					"new database entry\n");
+			elog(LOG, "PGSTAT: failed to initialize hash table for "
+				 "new database entry");
 			exit(1);
 		}
 	}
@@ -1846,8 +1858,7 @@ pgstat_sub_backend(int procpid)
 													 HASH_ENTER, &found);
 			if (deadbe == NULL)
 			{
-				fprintf(stderr, "PGSTAT: dead backend hash table out of memory "
-						"- abort\n");
+				elog(LOG, "PGSTAT: dead backend hash table out of memory");
 				exit(1);
 			}
 			if (!found)
@@ -1894,9 +1905,8 @@ pgstat_write_statsfile(void)
 	fpout = fopen(pgStat_tmpfname, PG_BINARY_W);
 	if (fpout == NULL)
 	{
-		fprintf(stderr, "PGSTAT: cannot open temp stats file\nPGSTAT: ");
-		perror(pgStat_tmpfname);
-		fflush(stderr);
+		elog(LOG, "PGSTAT: cannot open temp stats file %s: %m",
+			 pgStat_tmpfname);
 		return;
 	}
 
@@ -1921,8 +1931,8 @@ pgstat_write_statsfile(void)
 								(void *) &(dbentry->databaseid),
 								HASH_REMOVE, NULL) == NULL)
 				{
-					fprintf(stderr, "PGSTAT: database hash table corrupted "
-							"during cleanup - abort\n");
+					elog(LOG, "PGSTAT: database hash table corrupted "
+						 "during cleanup - abort");
 					exit(1);
 				}
 			}
@@ -1957,10 +1967,10 @@ pgstat_write_statsfile(void)
 									(void *) &(tabentry->tableid),
 									HASH_REMOVE, NULL) == NULL)
 					{
-						fprintf(stderr, "PGSTAT: tables hash table for "
-								"database %d corrupted during "
-								"cleanup - abort\n",
-								dbentry->databaseid);
+						elog(LOG, "PGSTAT: tables hash table for "
+							 "database %d corrupted during "
+							 "cleanup - abort",
+							 dbentry->databaseid);
 						exit(1);
 					}
 				}
@@ -2004,20 +2014,15 @@ pgstat_write_statsfile(void)
 	fputc('E', fpout);
 	if (fclose(fpout) < 0)
 	{
-		fprintf(stderr, "PGSTAT: Error closing temp stats file\nPGSTAT: ");
-		perror(pgStat_tmpfname);
-		fprintf(stderr, "PGSTAT: Abort\n");
-		fflush(stderr);
-		exit(1);
+		elog(LOG, "PGSTAT: Error closing temp stats file %s: %m",
+			 pgStat_tmpfname);
 	}
 	else
 	{
 		if (rename(pgStat_tmpfname, pgStat_fname) < 0)
 		{
-			fprintf(stderr, "PGSTAT: Cannot rename temp stats file\n"
-					"PGSTAT: ");
-			perror(pgStat_fname);
-			fflush(stderr);
+			elog(LOG, "PGSTAT: Cannot rename temp stats file %s: %m",
+				 pgStat_fname);
 		}
 	}
 
@@ -2037,8 +2042,8 @@ pgstat_write_statsfile(void)
 							(void *) &(deadbe->procpid),
 							HASH_REMOVE, NULL) == NULL)
 			{
-				fprintf(stderr, "PGSTAT: dead backend hash table corrupted "
-						"during cleanup - abort\n");
+				elog(LOG, "PGSTAT: dead backend hash table corrupted "
+					 "during cleanup - abort");
 				exit(1);
 			}
 		}
@@ -2102,9 +2107,10 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 	{
 		if (pgStatRunningInCollector)
 		{
-			fprintf(stderr, "PGSTAT: Creation of DB hash table failed\n");
+			elog(LOG, "PGSTAT: Creation of DB hash table failed");
 			exit(1);
 		}
+		/* in backend, can do normal error */
 		elog(ERROR, "PGSTAT: Creation of DB hash table failed");
 	}
 
@@ -2143,15 +2149,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2167,7 +2171,7 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr, "PGSTAT: DB hash table out of memory\n");
+						elog(LOG, "PGSTAT: DB hash table out of memory");
 						exit(1);
 					}
 					else
@@ -2180,15 +2184,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2219,15 +2221,15 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr, "PGSTAT: failed to initialize "
-								"hash table for new database entry\n");
+						elog(LOG, "PGSTAT: failed to initialize "
+							 "hash table for new database entry");
 						exit(1);
 					}
 					else
 					{
 						fclose(fpin);
 						elog(ERROR, "PGSTAT: failed to initialize "
-							 "hash table for new database entry\n");
+							 "hash table for new database entry");
 					}
 				}
 
@@ -2253,15 +2255,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2280,7 +2280,7 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr, "PGSTAT: Tab hash table out of memory\n");
+						elog(LOG, "PGSTAT: Tab hash table out of memory");
 						exit(1);
 					}
 					else
@@ -2294,15 +2294,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2325,15 +2323,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2381,15 +2377,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 				{
 					if (pgStatRunningInCollector)
 					{
-						fprintf(stderr,
-								"PGSTAT: corrupted pgstat.stat file\n");
+						elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
 					else
 					{
-						elog(NOTICE,
-							 "PGSTAT: corrupted pgstat.stat file");
+						elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 						fclose(fpin);
 						return;
 					}
@@ -2424,13 +2418,13 @@ pgstat_read_statsfile(HTAB **dbhash, Oid onlydb,
 			default:
 				if (pgStatRunningInCollector)
 				{
-					fprintf(stderr, "PGSTAT: corrupted pgstat.stat file\n");
+					elog(LOG, "PGSTAT: corrupted pgstat.stat file");
 					fclose(fpin);
 					return;
 				}
 				else
 				{
-					elog(NOTICE, "PGSTAT: corrupted pgstat.stat file");
+					elog(WARNING, "PGSTAT: corrupted pgstat.stat file");
 					fclose(fpin);
 					return;
 				}
@@ -2539,8 +2533,8 @@ pgstat_recv_tabstat(PgStat_MsgTabstat *msg, int len)
 													 HASH_ENTER, &found);
 		if (tabentry == NULL)
 		{
-			fprintf(stderr, "PGSTAT: tables hash table out of memory for "
-					"database %d - abort\n", dbentry->databaseid);
+			elog(LOG, "PGSTAT: tables hash table out of memory for "
+				 "database %d - abort", dbentry->databaseid);
 			exit(1);
 		}
 
@@ -2719,8 +2713,8 @@ pgstat_recv_resetcounter(PgStat_MsgResetcounter *msg, int len)
 								  HASH_ELEM | HASH_FUNCTION);
 	if (dbentry->tables == NULL)
 	{
-		fprintf(stderr, "PGSTAT: failed to reinitialize hash table for "
-				"database entry\n");
+		elog(LOG, "PGSTAT: failed to reinitialize hash table for "
+			 "database entry");
 		exit(1);
 	}
 }
