@@ -20,15 +20,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_files.c,v 1.14 2001/10/25 05:49:52 momjian Exp $
- *
- * Modifications - 28-Jun-2000 - pjw@rhyme.com.au
- *
- *	Initial version.
- *
- * Modifications - 04-Jan-2001 - pjw@rhyme.com.au
- *
- *	  - Check results of IO routines more carefully.
+ *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_files.c,v 1.21 2002/10/25 01:33:17 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -38,12 +30,12 @@
 
 static void _ArchiveEntry(ArchiveHandle *AH, TocEntry *te);
 static void _StartData(ArchiveHandle *AH, TocEntry *te);
-static int	_WriteData(ArchiveHandle *AH, const void *data, int dLen);
+static size_t _WriteData(ArchiveHandle *AH, const void *data, size_t dLen);
 static void _EndData(ArchiveHandle *AH, TocEntry *te);
 static int	_WriteByte(ArchiveHandle *AH, const int i);
 static int	_ReadByte(ArchiveHandle *);
-static int	_WriteBuf(ArchiveHandle *AH, const void *buf, int len);
-static int	_ReadBuf(ArchiveHandle *AH, void *buf, int len);
+static size_t _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len);
+static size_t _ReadBuf(ArchiveHandle *AH, void *buf, size_t len);
 static void _CloseArchive(ArchiveHandle *AH);
 static void _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt);
 static void _WriteExtraToc(ArchiveHandle *AH, TocEntry *te);
@@ -60,7 +52,7 @@ static void _EndBlobs(ArchiveHandle *AH, TocEntry *te);
 typedef struct
 {
 	int			hasSeek;
-	int			filePos;
+	off_t		filePos;
 	FILE	   *blobToc;
 } lclContext;
 
@@ -76,7 +68,7 @@ typedef struct
 
 static char *modulename = gettext_noop("file archiver");
 static void _LoadBlobs(ArchiveHandle *AH, RestoreOptions *ropt);
-static void _getBlobTocEntry(ArchiveHandle *AH, int *oid, char *fname);
+static void _getBlobTocEntry(ArchiveHandle *AH, Oid *oid, char *fname);
 
 /*
  *	Initializer
@@ -113,6 +105,12 @@ InitArchiveFmt_Files(ArchiveHandle *AH)
 	AH->formatData = (void *) ctx;
 	ctx->filePos = 0;
 
+	/* Initialize LO buffering */
+	AH->lo_buf_size = LOBBUFSIZE;
+	AH->lo_buf = (void *) malloc(LOBBUFSIZE);
+	if (AH->lo_buf == NULL)
+		die_horribly(AH, modulename, "out of memory\n");
+
 	/*
 	 * Now open the TOC file
 	 */
@@ -131,7 +129,7 @@ InitArchiveFmt_Files(ArchiveHandle *AH)
 		if (AH->FH == NULL)
 			die_horribly(NULL, modulename, "could not open output file: %s\n", strerror(errno));
 
-		ctx->hasSeek = (fseek(AH->FH, 0, SEEK_CUR) == 0);
+		ctx->hasSeek = checkSeek(AH->FH);
 
 		if (AH->compression < 0 || AH->compression > 9)
 			AH->compression = Z_DEFAULT_COMPRESSION;
@@ -149,7 +147,7 @@ InitArchiveFmt_Files(ArchiveHandle *AH)
 		if (AH->FH == NULL)
 			die_horribly(NULL, modulename, "could not open input file: %s\n", strerror(errno));
 
-		ctx->hasSeek = (fseek(AH->FH, 0, SEEK_CUR) == 0);
+		ctx->hasSeek = checkSeek(AH->FH);
 
 		ReadHead(AH);
 		ReadToc(AH);
@@ -157,7 +155,6 @@ InitArchiveFmt_Files(ArchiveHandle *AH)
 		if (fclose(AH->FH) != 0)
 			die_horribly(AH, modulename, "could not close TOC file: %s\n", strerror(errno));
 	}
-
 }
 
 /*
@@ -246,11 +243,10 @@ _StartData(ArchiveHandle *AH, TocEntry *te)
 
 	if (tctx->FH == NULL)
 		die_horribly(AH, modulename, "could not open data file for output\n");
-
 }
 
-static int
-_WriteData(ArchiveHandle *AH, const void *data, int dLen)
+static size_t
+_WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 {
 	lclTocEntry *tctx = (lclTocEntry *) AH->currToc->formatData;
 
@@ -278,7 +274,7 @@ static void
 _PrintFileData(ArchiveHandle *AH, char *filename, RestoreOptions *ropt)
 {
 	char		buf[4096];
-	int			cnt;
+	size_t		cnt;
 
 	if (!filename)
 		return;
@@ -300,7 +296,6 @@ _PrintFileData(ArchiveHandle *AH, char *filename, RestoreOptions *ropt)
 
 	if (GZCLOSE(AH->FH) != 0)
 		die_horribly(AH, modulename, "could not close data file after reading\n");
-
 }
 
 
@@ -322,16 +317,16 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 }
 
 static void
-_getBlobTocEntry(ArchiveHandle *AH, int *oid, char fname[K_STD_BUF_SIZE])
+_getBlobTocEntry(ArchiveHandle *AH, Oid *oid, char fname[K_STD_BUF_SIZE])
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	char		blobTe[K_STD_BUF_SIZE];
-	int			fpos;
-	int			eos;
+	size_t		fpos;
+	size_t		eos;
 
 	if (fgets(&blobTe[0], K_STD_BUF_SIZE - 1, ctx->blobToc) != NULL)
 	{
-		*oid = atoi(blobTe);
+		*oid = atooid(blobTe);
 
 		fpos = strcspn(blobTe, " ");
 
@@ -354,7 +349,7 @@ _getBlobTocEntry(ArchiveHandle *AH, int *oid, char fname[K_STD_BUF_SIZE])
 static void
 _LoadBlobs(ArchiveHandle *AH, RestoreOptions *ropt)
 {
-	int			oid;
+	Oid			oid;
 	lclContext *ctx = (lclContext *) AH->formatData;
 	char		fname[K_STD_BUF_SIZE];
 
@@ -407,25 +402,25 @@ _ReadByte(ArchiveHandle *AH)
 	return res;
 }
 
-static int
-_WriteBuf(ArchiveHandle *AH, const void *buf, int len)
+static size_t
+_WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = fwrite(buf, 1, len, AH->FH);
 	if (res != len)
-		die_horribly(AH, modulename, "write error in _WriteBuf (%d != %d)\n", res, len);
+		die_horribly(AH, modulename, "write error in _WriteBuf (%lu != %lu)\n", (unsigned long) res, (unsigned long) len);
 
 	ctx->filePos += res;
 	return res;
 }
 
-static int
-_ReadBuf(ArchiveHandle *AH, void *buf, int len)
+static size_t
+_ReadBuf(ArchiveHandle *AH, void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = fread(buf, 1, len, AH->FH);
 	ctx->filePos += res;
@@ -475,7 +470,6 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 	if (ctx->blobToc == NULL)
 		die_horribly(AH, modulename,
 					 "could not open large object TOC for output: %s\n", strerror(errno));
-
 }
 
 /*
@@ -503,9 +497,9 @@ _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 		sfx = "";
 
 	sprintf(fmode, "wb%d", AH->compression);
-	sprintf(fname, "blob_%d.dat%s", oid, sfx);
+	sprintf(fname, "blob_%u.dat%s", oid, sfx);
 
-	fprintf(ctx->blobToc, "%d %s\n", oid, fname);
+	fprintf(ctx->blobToc, "%u %s\n", oid, fname);
 
 #ifdef HAVE_LIBZ
 	tctx->FH = gzopen(fname, fmode);
