@@ -16,7 +16,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_tar.c,v 1.32 2002/10/25 01:33:17 momjian Exp $
+ *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_tar.c,v 1.32.2.2 2003/02/01 19:29:26 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <unistd.h>
 
 static void _ArchiveEntry(ArchiveHandle *AH, TocEntry *te);
@@ -980,7 +981,7 @@ _tarChecksum(char *header)
 	return sum + 256;			/* Assume 8 blanks in checksum field */
 }
 
-int
+bool
 isValidTarHeader(char *header)
 {
 	int			sum;
@@ -988,7 +989,17 @@ isValidTarHeader(char *header)
 
 	sscanf(&header[148], "%8o", &sum);
 
-	return (sum == chk && strncmp(&header[257], "ustar  ", 7) == 0);
+	if (sum != chk)
+		return false;
+
+	/* POSIX format */
+	if (strncmp(&header[257], "ustar00", 7) == 0)
+		return true;
+	/* older format */
+	if (strncmp(&header[257], "ustar  ", 7) == 0)
+		return true;
+
+	return false;
 }
 
 /* Given the member, write the TAR header & copy the file */
