@@ -20,18 +20,22 @@
  * for transmitted ints, whereas the backend modules (as of this writing)
  * still handle either network or little-endian byte order.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/interfaces/libpq/fe-misc.c,v 1.48 2001/03/31 23:13:30 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpq/fe-misc.c,v 1.85 2002/10/24 23:35:55 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 
 #include "postgres_fe.h"
 
+#include <errno.h>
+#include <signal.h>
+#include <time.h>
+
 #ifdef WIN32
 #include "win32.h"
 #else
@@ -39,10 +43,6 @@
 #include <sys/time.h>
 #endif
 
-#include <errno.h>
-#include <signal.h>
-#include <time.h>
-
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
@@ -50,24 +50,22 @@
 #include "libpq-fe.h"
 #include "libpq-int.h"
 #include "pqsignal.h"
-
-#ifdef MULTIBYTE
 #include "mb/pg_wchar.h"
-#endif
-
 
 #define DONOTICE(conn,message) \
 	((*(conn)->noticeHook) ((conn)->noticeArg, (message)))
 
+static int	pqPutBytes(const char *s, size_t nbytes, PGconn *conn);
 
-/* --------------------------------------------------------------------- */
-/* pqGetc:
-   get a character from the connection
 
-   All these routines return 0 on success, EOF on error.
-   Note that for the Get routines, EOF only means there is not enough
-   data in the buffer, not that there is necessarily a hard error.
-*/
+/*
+ * pqGetc:
+ *	get a character from the connection
+ *
+ *	All these routines return 0 on success, EOF on error.
+ *	Note that for the Get routines, EOF only means there is not enough
+ *	data in the buffer, not that there is necessarily a hard error.
+ */
 int
 pqGetc(char *result, PGconn *conn)
 {
@@ -83,73 +81,116 @@ pqGetc(char *result, PGconn *conn)
 }
 
 
-/* --------------------------------------------------------------------- */
-/* pqPutBytes: local routine to write N bytes to the connection,
-   with buffering
+/*
+ * write 1 char to the connection
  */
-static int
-pqPutBytes(const char *s, size_t nbytes, PGconn *conn)
+int
+pqPutc(char c, PGconn *conn)
 {
-	size_t		avail = Max(conn->outBufSize - conn->outCount, 0);
+	if (pqPutBytes(&c, 1, conn) == EOF)
+		return EOF;
 
-	/*
-	 * if we are non-blocking and the send queue is too full to buffer
-	 * this request then try to flush some and return an error
-	 */
-	if (pqIsnonblocking(conn) && nbytes > avail && pqFlush(conn))
-	{
-
-		/*
-		 * even if the flush failed we may still have written some data,
-		 * recalculate the size of the send-queue relative to the amount
-		 * we have to send, we may be able to queue it afterall even
-		 * though it's not sent to the database it's ok, any routines that
-		 * check the data coming from the database better call pqFlush()
-		 * anyway.
-		 */
-		if (nbytes > Max(conn->outBufSize - conn->outCount, 0))
-		{
-			printfPQExpBuffer(&conn->errorMessage,
-						   "pqPutBytes --  pqFlush couldn't flush enough"
-						 " data: space available: %d, space needed %d\n",
-					  Max(conn->outBufSize - conn->outCount, 0), nbytes);
-			return EOF;
-		}
-		/* fixup avail for while loop */
-		avail = Max(conn->outBufSize - conn->outCount, 0);
-	}
-
-	/*
-	 * is the amount of data to be sent is larger than the size of the
-	 * output buffer then we must flush it to make more room.
-	 *
-	 * the code above will make sure the loop conditional is never true for
-	 * non-blocking connections
-	 */
-	while (nbytes > avail)
-	{
-		memcpy(conn->outBuffer + conn->outCount, s, avail);
-		conn->outCount += avail;
-		s += avail;
-		nbytes -= avail;
-		if (pqFlush(conn))
-			return EOF;
-		avail = conn->outBufSize;
-	}
-
-	memcpy(conn->outBuffer + conn->outCount, s, nbytes);
-	conn->outCount += nbytes;
+	if (conn->Pfdebug)
+		fprintf(conn->Pfdebug, "To backend> %c\n", c);
 
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqGets:
-   get a null-terminated string from the connection,
-   and store it in an expansible PQExpBuffer.
-   If we run out of memory, all of the string is still read,
-   but the excess characters are silently discarded.
-*/
+
+/*
+ * pqPutBytes: local routine to write N bytes to the connection,
+ * with buffering
+ */
+static int
+pqPutBytes(const char *s, size_t nbytes, PGconn *conn)
+{
+	/*
+	 * Strategy to handle blocking and non-blocking connections: Fill the
+	 * output buffer and flush it repeatedly until either all data has
+	 * been sent or is at least queued in the buffer.
+	 *
+	 * For non-blocking connections, grow the buffer if not all data fits
+	 * into it and the buffer can't be sent because the socket would
+	 * block.
+	 */
+
+	while (nbytes)
+	{
+		size_t		avail,
+					remaining;
+
+		/* fill the output buffer */
+		avail = Max(conn->outBufSize - conn->outCount, 0);
+		remaining = Min(avail, nbytes);
+		memcpy(conn->outBuffer + conn->outCount, s, remaining);
+		conn->outCount += remaining;
+		s += remaining;
+		nbytes -= remaining;
+
+		/*
+		 * if the data didn't fit completely into the buffer, try to flush
+		 * the buffer
+		 */
+		if (nbytes)
+		{
+			int			send_result = pqSendSome(conn);
+
+			/* if there were errors, report them */
+			if (send_result < 0)
+				return EOF;
+
+			/*
+			 * if not all data could be sent, increase the output buffer,
+			 * put the rest of s into it and return successfully. This
+			 * case will only happen in a non-blocking connection
+			 */
+			if (send_result > 0)
+			{
+				/*
+				 * try to grow the buffer. FIXME: The new size could be
+				 * chosen more intelligently.
+				 */
+				size_t		buflen = (size_t) conn->outCount + nbytes;
+
+				if (buflen > (size_t) conn->outBufSize)
+				{
+					char	   *newbuf = realloc(conn->outBuffer, buflen);
+
+					if (!newbuf)
+					{
+						/* realloc failed. Probably out of memory */
+						printfPQExpBuffer(&conn->errorMessage,
+						   "cannot allocate memory for output buffer\n");
+						return EOF;
+					}
+					conn->outBuffer = newbuf;
+					conn->outBufSize = buflen;
+				}
+				/* put the data into it */
+				memcpy(conn->outBuffer + conn->outCount, s, nbytes);
+				conn->outCount += nbytes;
+
+				/* report success. */
+				return 0;
+			}
+		}
+
+		/*
+		 * pqSendSome was able to send all data. Continue with the next
+		 * chunk of s.
+		 */
+	}							/* while */
+
+	return 0;
+}
+
+/*
+ * pqGets:
+ * get a null-terminated string from the connection,
+ * and store it in an expansible PQExpBuffer.
+ * If we run out of memory, all of the string is still read,
+ * but the excess characters are silently discarded.
+ */
 int
 pqGets(PQExpBuffer buf, PGconn *conn)
 {
@@ -179,7 +220,7 @@ pqGets(PQExpBuffer buf, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
+
 int
 pqPuts(const char *s, PGconn *conn)
 {
@@ -192,14 +233,14 @@ pqPuts(const char *s, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqGetnchar:
-   get a string of exactly len bytes in buffer s, no null termination
-*/
+/*
+ * pqGetnchar:
+ *	get a string of exactly len bytes in buffer s, no null termination
+ */
 int
 pqGetnchar(char *s, size_t len, PGconn *conn)
 {
-	if (len < 0 || len > conn->inEnd - conn->inCursor)
+	if (len < 0 || len > (size_t) (conn->inEnd - conn->inCursor))
 		return EOF;
 
 	memcpy(s, conn->inBuffer + conn->inCursor, len);
@@ -213,10 +254,10 @@ pqGetnchar(char *s, size_t len, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqPutnchar:
-   send a string of exactly len bytes, no null termination needed
-*/
+/*
+ * pqPutnchar:
+ *	send a string of exactly len bytes, no null termination needed
+ */
 int
 pqPutnchar(const char *s, size_t len, PGconn *conn)
 {
@@ -229,11 +270,11 @@ pqPutnchar(const char *s, size_t len, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pgGetInt
-   read a 2 or 4 byte integer and convert from network byte order
-   to local byte order
-*/
+/*
+ * pgGetInt
+ *	read a 2 or 4 byte integer and convert from network byte order
+ *	to local byte order
+ */
 int
 pqGetInt(int *result, size_t bytes, PGconn *conn)
 {
@@ -258,8 +299,9 @@ pqGetInt(int *result, size_t bytes, PGconn *conn)
 			*result = (int) ntohl(tmp4);
 			break;
 		default:
-			sprintf(noticeBuf,
-					"pqGetInt: int size %lu not supported\n", (unsigned long) bytes);
+			snprintf(noticeBuf, sizeof(noticeBuf),
+					 libpq_gettext("integer of size %lu not supported by pqGetInt\n"),
+					 (unsigned long) bytes);
 			DONOTICE(conn, noticeBuf);
 			return EOF;
 	}
@@ -270,11 +312,11 @@ pqGetInt(int *result, size_t bytes, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pgPutInt
-   send an integer of 2 or 4 bytes, converting from host byte order
-   to network byte order.
-*/
+/*
+ * pgPutInt
+ * send an integer of 2 or 4 bytes, converting from host byte order
+ * to network byte order.
+ */
 int
 pqPutInt(int value, size_t bytes, PGconn *conn)
 {
@@ -295,8 +337,9 @@ pqPutInt(int value, size_t bytes, PGconn *conn)
 				return EOF;
 			break;
 		default:
-			sprintf(noticeBuf,
-					"pqPutInt: int size %lu not supported\n", (unsigned long) bytes);
+			snprintf(noticeBuf, sizeof(noticeBuf),
+					 libpq_gettext("integer of size %lu not supported by pqPutInt\n"),
+					 (unsigned long) bytes);
 			DONOTICE(conn, noticeBuf);
 			return EOF;
 	}
@@ -307,8 +350,9 @@ pqPutInt(int value, size_t bytes, PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqReadReady: is select() saying the file is ready to read?
+/*
+ * pqReadReady: is select() saying the file is ready to read?
+ * JAB: -or- if SSL is enabled and used, is it buffering bytes?
  * Returns -1 on failure, 0 if not ready, 1 if ready.
  */
 int
@@ -320,7 +364,16 @@ pqReadReady(PGconn *conn)
 	if (!conn || conn->sock < 0)
 		return -1;
 
-retry:
+/* JAB: Check for SSL library buffering read bytes */
+#ifdef USE_SSL
+	if (conn->ssl && SSL_pending(conn->ssl) > 0)
+	{
+		/* short-circuit the select */
+		return 1;
+	}
+#endif
+
+retry1:
 	FD_ZERO(&input_mask);
 	FD_SET(conn->sock, &input_mask);
 	timeout.tv_sec = 0;
@@ -328,21 +381,21 @@ retry:
 	if (select(conn->sock + 1, &input_mask, (fd_set *) NULL, (fd_set *) NULL,
 			   &timeout) < 0)
 	{
-		if (errno == EINTR)
+		if (SOCK_ERRNO == EINTR)
 			/* Interrupted system call - we'll just try again */
-			goto retry;
+			goto retry1;
 
 		printfPQExpBuffer(&conn->errorMessage,
-					  "pqReadReady() -- select() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+						  libpq_gettext("select() failed: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return -1;
 	}
 
 	return FD_ISSET(conn->sock, &input_mask) ? 1 : 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqWriteReady: is select() saying the file is ready to write?
+/*
+ * pqWriteReady: is select() saying the file is ready to write?
  * Returns -1 on failure, 0 if not ready, 1 if ready.
  */
 int
@@ -354,7 +407,7 @@ pqWriteReady(PGconn *conn)
 	if (!conn || conn->sock < 0)
 		return -1;
 
-retry:
+retry2:
 	FD_ZERO(&input_mask);
 	FD_SET(conn->sock, &input_mask);
 	timeout.tv_sec = 0;
@@ -362,20 +415,20 @@ retry:
 	if (select(conn->sock + 1, (fd_set *) NULL, &input_mask, (fd_set *) NULL,
 			   &timeout) < 0)
 	{
-		if (errno == EINTR)
+		if (SOCK_ERRNO == EINTR)
 			/* Interrupted system call - we'll just try again */
-			goto retry;
+			goto retry2;
 
 		printfPQExpBuffer(&conn->errorMessage,
-					 "pqWriteReady() -- select() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+						  libpq_gettext("select() failed: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return -1;
 	}
 	return FD_ISSET(conn->sock, &input_mask) ? 1 : 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqReadData: read more data, if any is available
+/* ----------
+ * pqReadData: read more data, if any is available
  * Possible return values:
  *	 1: successfully loaded at least one more byte
  *	 0: no data is presently available, but no error detected
@@ -383,6 +436,7 @@ retry:
  *		conn->errorMessage set
  * NOTE: callers must not assume that pointers or indexes into conn->inBuffer
  * remain valid across this call!
+ * ----------
  */
 int
 pqReadData(PGconn *conn)
@@ -393,21 +447,27 @@ pqReadData(PGconn *conn)
 	if (conn->sock < 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "pqReadData() -- connection not open\n");
+						  libpq_gettext("connection not open\n"));
 		return -1;
 	}
 
 	/* Left-justify any data in the buffer to make room */
 	if (conn->inStart < conn->inEnd)
 	{
-		memmove(conn->inBuffer, conn->inBuffer + conn->inStart,
-				conn->inEnd - conn->inStart);
-		conn->inEnd -= conn->inStart;
-		conn->inCursor -= conn->inStart;
-		conn->inStart = 0;
+		if (conn->inStart > 0)
+		{
+			memmove(conn->inBuffer, conn->inBuffer + conn->inStart,
+					conn->inEnd - conn->inStart);
+			conn->inEnd -= conn->inStart;
+			conn->inCursor -= conn->inStart;
+			conn->inStart = 0;
+		}
 	}
 	else
+	{
+		/* buffer is logically empty, reset it */
 		conn->inStart = conn->inCursor = conn->inEnd = 0;
+	}
 
 	/*
 	 * If the buffer is fairly full, enlarge it. We need to be able to
@@ -430,36 +490,30 @@ pqReadData(PGconn *conn)
 	}
 
 	/* OK, try to read some data */
-tryAgain:
-#ifdef USE_SSL
-	if (conn->ssl)
-		nread = SSL_read(conn->ssl, conn->inBuffer + conn->inEnd,
-						 conn->inBufSize - conn->inEnd);
-	else
-#endif
-		nread = recv(conn->sock, conn->inBuffer + conn->inEnd,
-					 conn->inBufSize - conn->inEnd, 0);
+retry3:
+	nread = pqsecure_read(conn, conn->inBuffer + conn->inEnd,
+						  conn->inBufSize - conn->inEnd);
 	if (nread < 0)
 	{
-		if (errno == EINTR)
-			goto tryAgain;
+		if (SOCK_ERRNO == EINTR)
+			goto retry3;
 		/* Some systems return EAGAIN/EWOULDBLOCK for no data */
 #ifdef EAGAIN
-		if (errno == EAGAIN)
+		if (SOCK_ERRNO == EAGAIN)
 			return someread;
 #endif
 #if defined(EWOULDBLOCK) && (!defined(EAGAIN) || (EWOULDBLOCK != EAGAIN))
-		if (errno == EWOULDBLOCK)
+		if (SOCK_ERRNO == EWOULDBLOCK)
 			return someread;
 #endif
 		/* We might get ECONNRESET here if using TCP and backend died */
 #ifdef ECONNRESET
-		if (errno == ECONNRESET)
+		if (SOCK_ERRNO == ECONNRESET)
 			goto definitelyFailed;
 #endif
 		printfPQExpBuffer(&conn->errorMessage,
-						"pqReadData() --  read() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+			   libpq_gettext("could not receive data from server: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return -1;
 	}
 	if (nread > 0)
@@ -483,7 +537,7 @@ tryAgain:
 			(conn->inBufSize - conn->inEnd) >= 8192)
 		{
 			someread = 1;
-			goto tryAgain;
+			goto retry3;
 		}
 		return 1;
 	}
@@ -516,36 +570,30 @@ tryAgain:
 	 * Still not sure that it's EOF, because some data could have just
 	 * arrived.
 	 */
-tryAgain2:
-#ifdef USE_SSL
-	if (conn->ssl)
-		nread = SSL_read(conn->ssl, conn->inBuffer + conn->inEnd,
-						 conn->inBufSize - conn->inEnd);
-	else
-#endif
-		nread = recv(conn->sock, conn->inBuffer + conn->inEnd,
-					 conn->inBufSize - conn->inEnd, 0);
+retry4:
+	nread = pqsecure_read(conn, conn->inBuffer + conn->inEnd,
+						  conn->inBufSize - conn->inEnd);
 	if (nread < 0)
 	{
-		if (errno == EINTR)
-			goto tryAgain2;
+		if (SOCK_ERRNO == EINTR)
+			goto retry4;
 		/* Some systems return EAGAIN/EWOULDBLOCK for no data */
 #ifdef EAGAIN
-		if (errno == EAGAIN)
+		if (SOCK_ERRNO == EAGAIN)
 			return 0;
 #endif
 #if defined(EWOULDBLOCK) && (!defined(EAGAIN) || (EWOULDBLOCK != EAGAIN))
-		if (errno == EWOULDBLOCK)
+		if (SOCK_ERRNO == EWOULDBLOCK)
 			return 0;
 #endif
 		/* We might get ECONNRESET here if using TCP and backend died */
 #ifdef ECONNRESET
-		if (errno == ECONNRESET)
+		if (SOCK_ERRNO == ECONNRESET)
 			goto definitelyFailed;
 #endif
 		printfPQExpBuffer(&conn->errorMessage,
-						"pqReadData() --  read() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+			   libpq_gettext("could not receive data from server: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return -1;
 	}
 	if (nread > 0)
@@ -560,10 +608,12 @@ tryAgain2:
 	 */
 definitelyFailed:
 	printfPQExpBuffer(&conn->errorMessage,
-			 "pqReadData() -- backend closed the channel unexpectedly.\n"
-			  "\tThis probably means the backend terminated abnormally\n"
-					  "\tbefore or while processing the request.\n");
+					  libpq_gettext(
+							"server closed the connection unexpectedly\n"
+			   "\tThis probably means the server terminated abnormally\n"
+						 "\tbefore or while processing the request.\n"));
 	conn->status = CONNECTION_BAD;		/* No more connection to backend */
+	pqsecure_close(conn);
 #ifdef WIN32
 	closesocket(conn->sock);
 #else
@@ -574,11 +624,14 @@ definitelyFailed:
 	return -1;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqFlush: send any data waiting in the output buffer
+/*
+ * pqSendSome: send any data waiting in the output buffer.
+ *
+ * Return 0 on sucess, -1 on failure and 1 when data remains because the
+ * socket would block and the connection is non-blocking.
  */
 int
-pqFlush(PGconn *conn)
+pqSendSome(PGconn *conn)
 {
 	char	   *ptr = conn->outBuffer;
 	int			len = conn->outCount;
@@ -586,8 +639,8 @@ pqFlush(PGconn *conn)
 	if (conn->sock < 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "pqFlush() -- connection not open\n");
-		return EOF;
+						  libpq_gettext("connection not open\n"));
+		return -1;
 	}
 
 	/*
@@ -600,34 +653,18 @@ pqFlush(PGconn *conn)
 	/* while there's still data to send */
 	while (len > 0)
 	{
-		/* Prevent being SIGPIPEd if backend has closed the connection. */
-#ifndef WIN32
-		pqsigfunc	oldsighandler = pqsignal(SIGPIPE, SIG_IGN);
-
-#endif
-
 		int			sent;
 
-#ifdef USE_SSL
-		if (conn->ssl)
-			sent = SSL_write(conn->ssl, ptr, len);
-		else
-#endif
-			sent = send(conn->sock, ptr, len, 0);
-
-#ifndef WIN32
-		pqsignal(SIGPIPE, oldsighandler);
-#endif
+		sent = pqsecure_write(conn, ptr, len);
 
 		if (sent < 0)
 		{
-
 			/*
 			 * Anything except EAGAIN or EWOULDBLOCK is trouble. If it's
 			 * EPIPE or ECONNRESET, assume we've lost the backend
 			 * connection permanently.
 			 */
-			switch (errno)
+			switch (SOCK_ERRNO)
 			{
 #ifdef EAGAIN
 				case EAGAIN:
@@ -645,26 +682,27 @@ pqFlush(PGconn *conn)
 				case ECONNRESET:
 #endif
 					printfPQExpBuffer(&conn->errorMessage,
-									  "pqFlush() -- backend closed the channel unexpectedly.\n"
-									  "\tThis probably means the backend terminated abnormally"
-						   " before or while processing the request.\n");
+									  libpq_gettext(
+							"server closed the connection unexpectedly\n"
+													"\tThis probably means the server terminated abnormally\n"
+						 "\tbefore or while processing the request.\n"));
 
 					/*
 					 * We used to close the socket here, but that's a bad
 					 * idea since there might be unread data waiting
-					 * (typically, a NOTICE message from the backend
+					 * (typically, a WARNING message from the backend
 					 * telling us it's committing hara-kiri...).  Leave
 					 * the socket open until pqReadData finds no more data
 					 * can be read.
 					 */
-					return EOF;
+					return -1;
 
 				default:
 					printfPQExpBuffer(&conn->errorMessage,
-					  "pqFlush() --  couldn't send data: errno=%d\n%s\n",
-									  errno, strerror(errno));
+					libpq_gettext("could not send data to server: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
 					/* We don't assume it's a fatal error... */
-					return EOF;
+					return -1;
 			}
 		}
 		else
@@ -679,7 +717,7 @@ pqFlush(PGconn *conn)
 
 			/*
 			 * if the socket is in non-blocking mode we may need to abort
-			 * here
+			 * here and return 1 to indicate that data is still pending.
 			 */
 #ifdef USE_SSL
 			/* can't do anything for our SSL users yet */
@@ -691,14 +729,14 @@ pqFlush(PGconn *conn)
 					/* shift the contents of the buffer */
 					memmove(conn->outBuffer, ptr, len);
 					conn->outCount = len;
-					return EOF;
+					return 1;
 				}
 #ifdef USE_SSL
 			}
 #endif
 
 			if (pqWait(FALSE, TRUE, conn))
-				return EOF;
+				return -1;
 		}
 	}
 
@@ -710,8 +748,29 @@ pqFlush(PGconn *conn)
 	return 0;
 }
 
-/* --------------------------------------------------------------------- */
-/* pqWait: wait until we can read or write the connection socket
+
+
+/*
+ * pqFlush: send any data waiting in the output buffer
+ *
+ * Implemented in terms of pqSendSome to recreate the old behavior which
+ * returned 0 if all data was sent or EOF. EOF was sent regardless of
+ * whether an error occurred or not all data was sent on a non-blocking
+ * socket.
+ */
+int
+pqFlush(PGconn *conn)
+{
+	if (pqSendSome(conn))
+		return EOF;
+	return 0;
+}
+
+/*
+ * pqWait: wait until we can read or write the connection socket
+ *
+ * JAB: If SSL enabled and used and forRead, buffered bytes short-circuit the
+ * call to select().
  *
  * We also stop waiting and return if the kernel flags an exception condition
  * on the socket.  The actual error condition will be detected and reported
@@ -719,21 +778,48 @@ pqFlush(PGconn *conn)
  */
 int
 pqWait(int forRead, int forWrite, PGconn *conn)
+{
+	return pqWaitTimed(forRead, forWrite, conn, (time_t) -1);
+}
+
+/*
+ * pqWaitTimed: wait, but not past finish_time.
+ *
+ * If finish_time is exceeded then we return failure (EOF).  This is different
+ * from the response for a kernel exception (return 0) because we don't want
+ * the caller to try to read/write in that case.
+ *
+ * finish_time = ((time_t) -1) disables the wait limit.
+ */
+int
+pqWaitTimed(int forRead, int forWrite, PGconn *conn, time_t finish_time)
 {
 	fd_set		input_mask;
 	fd_set		output_mask;
 	fd_set		except_mask;
+	struct timeval tmp_timeout;
+	struct timeval *ptmp_timeout = NULL;
+	int			selresult;
 
 	if (conn->sock < 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "pqWait() -- connection not open\n");
+						  libpq_gettext("connection not open\n"));
 		return EOF;
 	}
 
+/* JAB: Check for SSL library buffering read bytes */
+#ifdef USE_SSL
+	if (forRead && conn->ssl && SSL_pending(conn->ssl) > 0)
+	{
+		/* short-circuit the select */
+		return 0;
+	}
+#endif
+
 	if (forRead || forWrite)
 	{
-retry:
+retry5:
 		FD_ZERO(&input_mask);
 		FD_ZERO(&output_mask);
 		FD_ZERO(&except_mask);
@@ -742,14 +828,39 @@ retry:
 		if (forWrite)
 			FD_SET(conn->sock, &output_mask);
 		FD_SET(conn->sock, &except_mask);
-		if (select(conn->sock + 1, &input_mask, &output_mask, &except_mask,
-				   (struct timeval *) NULL) < 0)
+
+		if (finish_time != ((time_t) -1))
 		{
-			if (errno == EINTR)
-				goto retry;
+			/*
+			 * Set up delay.  Assume caller incremented finish_time
+			 * so that we can error out as soon as time() passes it.
+			 * Note we will recalculate delay each time through the loop.
+			 */
+			time_t	now = time(NULL);
+
+			if (finish_time > now)
+				tmp_timeout.tv_sec = finish_time - now;
+			else
+				tmp_timeout.tv_sec = 0;
+			tmp_timeout.tv_usec = 0;
+			ptmp_timeout = &tmp_timeout;
+		}
+
+		selresult = select(conn->sock + 1, &input_mask, &output_mask,
+						   &except_mask, ptmp_timeout);
+		if (selresult < 0)
+		{
+			if (SOCK_ERRNO == EINTR)
+				goto retry5;
 			printfPQExpBuffer(&conn->errorMessage,
-							  "pqWait() -- select() failed: errno=%d\n%s\n",
-							  errno, strerror(errno));
+							  libpq_gettext("select() failed: %s\n"),
+							  SOCK_STRERROR(SOCK_ERRNO));
+			return EOF;
+		}
+		if (selresult == 0)
+		{
+			printfPQExpBuffer(&conn->errorMessage,
+							  libpq_gettext("timeout expired\n"));
 			return EOF;
 		}
 	}
@@ -764,7 +875,6 @@ retry:
  * to be in fe-print.c but that file is doomed.
  */
 
-#ifdef MULTIBYTE
 /*
  * returns the byte length of the word beginning s, using the
  * specified encoding.
@@ -782,7 +892,7 @@ int
 PQenv2encoding(void)
 {
 	char	   *str;
-	int			encoding = SQL_ASCII;
+	int			encoding = PG_SQL_ASCII;
 
 	str = getenv("PGCLIENTENCODING");
 	if (str && *str != '\0')
@@ -790,20 +900,20 @@ PQenv2encoding(void)
 	return (encoding);
 }
 
-#else
 
-/* Provide a default definition in case someone calls it anyway */
-int
-PQmblen(const unsigned char *s, int encoding)
+#ifdef ENABLE_NLS
+char *
+libpq_gettext(const char *msgid)
 {
-	(void) s;
-	(void) encoding;
-	return 1;
-}
-int
-PQenv2encoding(void)
-{
-	return 0;
+	static int	already_bound = 0;
+
+	if (!already_bound)
+	{
+		already_bound = 1;
+		bindtextdomain("libpq", LOCALEDIR);
+	}
+
+	return dgettext("libpq", msgid);
 }
 
-#endif	 /* MULTIBYTE */
+#endif   /* ENABLE_NLS */
