@@ -11,7 +11,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpq/fe-secure.c,v 1.15.2.2 2002/12/18 13:20:03 pgsql Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpq/fe-secure.c,v 1.15.2.5 2003/04/10 23:03:13 tgl Exp $
  *
  * NOTES
  *	  The client *requires* a valid server certificate.  Since
@@ -262,17 +262,24 @@ pqsecure_read(PGconn *conn, void *ptr, size_t len)
 #ifdef USE_SSL
 	if (conn->ssl)
 	{
+	rloop:
 		n = SSL_read(conn->ssl, ptr, len);
 		switch (SSL_get_error(conn->ssl, n))
 		{
 			case SSL_ERROR_NONE:
 				break;
 			case SSL_ERROR_WANT_READ:
-				break;
+			case SSL_ERROR_WANT_WRITE:
+				/* XXX to support nonblock I/O, we should return 0 here */
+				goto rloop;
 			case SSL_ERROR_SYSCALL:
-				printfPQExpBuffer(&conn->errorMessage,
-								libpq_gettext("SSL SYSCALL error: %s\n"),
-								  SOCK_STRERROR(SOCK_ERRNO));
+				if (n == -1)
+					printfPQExpBuffer(&conn->errorMessage,
+									  libpq_gettext("SSL SYSCALL error: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
+				else
+					printfPQExpBuffer(&conn->errorMessage,
+									  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
 				break;
 			case SSL_ERROR_SSL:
 				printfPQExpBuffer(&conn->errorMessage,
@@ -283,6 +290,10 @@ pqsecure_read(PGconn *conn, void *ptr, size_t len)
 				SOCK_ERRNO = ECONNRESET;
 				n = -1;
 				break;
+			default:
+				printfPQExpBuffer(&conn->errorMessage,
+								  libpq_gettext("Unknown SSL error code\n"));
+				break;
 		}
 	}
 	else
@@ -307,17 +318,24 @@ pqsecure_write(PGconn *conn, const void *ptr, size_t len)
 #ifdef USE_SSL
 	if (conn->ssl)
 	{
+	wloop:
 		n = SSL_write(conn->ssl, ptr, len);
 		switch (SSL_get_error(conn->ssl, n))
 		{
 			case SSL_ERROR_NONE:
 				break;
+			case SSL_ERROR_WANT_READ:
 			case SSL_ERROR_WANT_WRITE:
-				break;
+				/* XXX to support nonblock I/O, we should return 0 here */
+				goto wloop;
 			case SSL_ERROR_SYSCALL:
-				printfPQExpBuffer(&conn->errorMessage,
-								libpq_gettext("SSL SYSCALL error: %s\n"),
-								  SOCK_STRERROR(SOCK_ERRNO));
+				if (n == -1)
+					printfPQExpBuffer(&conn->errorMessage,
+									  libpq_gettext("SSL SYSCALL error: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
+				else
+					printfPQExpBuffer(&conn->errorMessage,
+									  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
 				break;
 			case SSL_ERROR_SSL:
 				printfPQExpBuffer(&conn->errorMessage,
@@ -328,6 +346,10 @@ pqsecure_write(PGconn *conn, const void *ptr, size_t len)
 				SOCK_ERRNO = ECONNRESET;
 				n = -1;
 				break;
+			default:
+				printfPQExpBuffer(&conn->errorMessage,
+								  libpq_gettext("Unknown SSL error code\n"));
+				break;
 		}
 	}
 	else
@@ -712,7 +734,7 @@ initialize_SSL(PGconn *conn)
 	{
 		SSL_library_init();
 		SSL_load_error_strings();
-		SSL_context = SSL_CTX_new(SSLv23_method());
+		SSL_context = SSL_CTX_new(TLSv1_method());
 		if (!SSL_context)
 		{
 			printfPQExpBuffer(&conn->errorMessage,
